#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir()
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
maketar=${2}
if [[ "$extraqual" == "tar" ]]; then
  unset extraqual
  maketar=tar
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=catch
origpkgver=v2_4_0
pkgver=${origpkgver//-/_}
ssibuildshims_version=v1_04_09
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtardir=${package}-${pkgdotver}
pkgtarname=${pkgtardir}.tar.bz2

get_this_dir
get_ssibuildshims

set -- # Clear positional parameters.

source define_basics NO_QUAL NO_FQ_DIR

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/include ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball -e lib64 -F ${product_dir} ${package} ${pkgver} NO_FLAVOR
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
[[ -f ups/${package}.table ]] || ssi_die "Unable to find UPS table file in source tarball"

fakedb=${product_dir}/${package}/${pkgver}/fakedb

${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} NO_FLAVOR

setup -B ${package} ${pkgver} -z ${fakedb}:${product_dir}:${PRODUCTS} || \
  ssi_die "fake setup failed"

ups active

set -x
(( want_tests = ${WANT_TESTS:-0} ))

if [[ "${OS1}" == Darwin ]]; then
  echo "Using native cc/c++ for tests"
  export CXX=c++
  export CC=cc
else
    gccprod=$(latest_version -t ${product_dir} --ge 6.1.0 gcc -z "${product_dir}" -z "${PRODUCTS}")
    if [[ -n "${gccprod}" ]]; then
	echo "${gccprod} is available for use with tests"
	eval setup -B ${gccprod} -z "${product_dir}" -z "${PRODUCTS}" || \
	    ssi_die "Unable to set up ${gccprod}"
	export CXX=g++
	export CC=gcc
    else
	echo "Modern (GCC >= 6.1) is not available. Tests will be skipped."
    fi
fi

ups active
set +x
# Untar source
cd "${sourcedir}" && rm -rf ${pkgtardir} || ssi_die "Unable to cd into ${sourcedir}"
tar xf ${tardir}/${pkgtarname}* || ssi_die "Unable to untar source"

# Apply patches
cd ${package}-${pkgdotver} || ssi_die "Unable to cd into ${package}-${pkgdotver} for patch application."

patches=(${patchdir}/AddTargets.patch)

for patch in "${patches[@]}"; do
  [[ -r "${patch}" ]] || continue # Don't care if there isn't one this time.
  if grep -e '^\(---\|+++\) \([ab]\|'"${package}"'-[^/]*\)/' ${patch} >/dev/null 2>&1; then
    pval=1
  else
    pval=0
  fi
  patch ${DEBUG_PATCHES:+--verbose} -p${pval} -i "${patch}" || ssi_die "application of patch ${patch} failed."
done

[[ -n "${DEBUG_PATCHES}" ]] && exit 1

source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake "${product_dir}"|| ssi_die "failed to setup cmake"

cd "${blddir}" || ssi_die "Unable to cd into ${blddir}"

set -x

# Testing purposes
${CXX} -xc++ -std=c++14 -c /dev/null >/dev/null 2>&1 && cppdef=-DUSE_CPP14=ON
[[ -n "${cppdef}" ]] || g++ -xc++ -std=c++11 -c /dev/null >/dev/null 2>&1 || \
  ssi_die "Catch2 requires C++11 or better."

cmake -DCMAKE_INSTALL_PREFIX=${product_dir}/${package}/${pkgver} \
  ${cppdef} \
  "${sourcedir}"/${pkgtardir} || \
  ssi_die "configure failed"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

make -j ${ncore} || ssi_die "make failed"
(( ${want_tests:-0} )) && { ctest -j ${ncore} || ssi_die "self-tests failed"; }
make -j ${ncore} install || ssi_die "make install failed"

# Install contributed CMake file.
install -d ${pkgdir}/share/cmake/catch
install -m 0644 ${sourcedir}/${pkgtardir}/contrib/*.cmake ${pkgdir}/share/cmake/catch

set +x

if [ ! -d ${pkgdir}/include ]
then
   echo "ERROR: failed to create ${pkgdir}/include"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} NO_FLAVOR || \
  ssi_die "failed to declare ${package} ${pkgver}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
unsetup ${package}
setup -B ${package} ${pkgver} -z ${product_dir}:${PRODUCTS}

PACKAGE=$(echo $package | tr '[a-z]' '[A-Z]')
echo "$package is installed at $(eval echo "\${${PACKAGE}_DIR}")"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/include ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball -e lib64 -F ${product_dir} ${package} ${pkgver} NO_FLAVOR
fi

exit 0
